"""
Used by setup.py, code.py
It is semantic-enough, major/minor then graymeta api release
major.minor.graymeta-compatible-point-release
I put -dev on the very end to indicate this code is not published
I take -dev off the end, then publish, then increment the minor version, put dev
on the end and commit again
"""
VERSION='1.17.30'
